﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SFML.Graphics;
using SFML.Window;
using TwelvePx.Interfaces;

namespace TwelvePx.Core
{
    public class GameEngine
    {
        public RenderWindow Window { get; private set; }
        public IScreen Screen { get; set; }
        public bool IsRunning { get; set; }

        private DateTime lastUpdate;

        public GameEngine()
            : this(null)
        {
        }

        public GameEngine(IScreen StartScreen)
        {
            // Set up window
            Window = new RenderWindow(new VideoMode(800, 600), "TwelvePx", Styles.Close);
            Window.SetView(new View(new FloatRect(0, 0, 4, 3)));
            Window.SetFramerateLimit(25);
            Window.Size = new Vector2u(800, 600);

            // Attach event handlers
            Window.KeyPressed += new EventHandler<KeyEventArgs>(Window_OnKeyPress);
            Window.KeyReleased += new EventHandler<KeyEventArgs>(Window_OnKeyRelease);
            Window.Closed += new EventHandler(Window_OnClose);
            Window.Resized += new EventHandler<SizeEventArgs>(Window_Resized);
            Window.GainedFocus += new EventHandler(Window_GainedFocus);
            Window.LostFocus += new EventHandler(Window_LostFocus);
            Window.TextEntered += new EventHandler<TextEventArgs>(Window_TextEntered);
            Window.MouseMoved += new EventHandler<MouseMoveEventArgs>(Window_MouseMoved);
            Window.MouseEntered += new EventHandler(Window_MouseEntered);
            Window.MouseLeft += new EventHandler(Window_MouseLeft);
            Window.MouseButtonPressed += new EventHandler<MouseButtonEventArgs>(Window_MouseButtonPressed);
            Window.MouseButtonReleased += new EventHandler<MouseButtonEventArgs>(Window_MouseButtonReleased);

            // Set up game vars
            IsRunning = true;

            Screen = StartScreen;
        }

        public void Run()
        {
            lastUpdate = DateTime.Now;

            while (Window.IsOpen() && IsRunning && Screen != null)
            {
                // Update the game elements and update last update var
                Screen.Update((float)(DateTime.Now - lastUpdate).TotalSeconds);
                lastUpdate = DateTime.Now;

                // Render all game elements
                Window.Draw(Screen);

                // Display everything
                Window.Display();

                // Handle events
                Window.DispatchEvents();
            }
        }

        public void Destroy()
        {
            IsRunning = false;
            if (Window != null)
            {
                if (Window.IsOpen())
                    Window.Close();
            }
            Screen = null;
        }

        // Event handlers
        #region "Event Handlers"

        private void Window_OnKeyPress(object sender, KeyEventArgs args)
        {
            bool handled = false;
            Screen.OnKeyPress(args, ref handled);
        }

        private void Window_OnKeyRelease(object sender, KeyEventArgs args)
        {
            bool handled = false;
            Screen.OnKeyRelease(args, ref handled);
        }

        private void Window_OnClose(object sender, EventArgs args)
        {
            bool cancel = false;
            Screen.OnClose(ref cancel);
            if (!cancel)
            {
                this.Destroy();
            }
        }

        private void Window_Resized(object sender, SizeEventArgs args)
        {
            Screen.OnResize(args);
        }

        private void Window_GainedFocus(object sender, EventArgs args)
        {
            Screen.OnFocusGain();
        }

        private void Window_LostFocus(object sender, EventArgs args)
        {
            Screen.OnFocusLose();
        }

        private void Window_TextEntered(object sender, TextEventArgs args)
        {
            bool handled = false;
            Screen.OnTextInput(args, ref handled);
        }

        private void Window_MouseMoved(object sender, MouseMoveEventArgs args)
        {
            bool handled = false;
            Screen.OnMouseMove(args, ref handled);
        }

        private void Window_MouseEntered(object sender, EventArgs args)
        {
            Screen.OnMouseEnter();
        }

        private void Window_MouseLeft(object sender, EventArgs args)
        {
            Screen.OnMouseLeave();
        }

        private void Window_MouseButtonPressed(object sender, MouseButtonEventArgs args)
        {
            bool handled = false;
            Screen.OnMouseButtonPress(args, ref handled);
        }

        private void Window_MouseButtonReleased(object sender, MouseButtonEventArgs args)
        {
            bool handled = false;
            Screen.OnMouseButtonRelease(args, ref handled);
        }

        #endregion
    }
}
